import time
from typing import List
from typing import Optional
from urllib.parse import urljoin

from bs4 import BeautifulSoup
from langchain.schema import Document
from playwright.sync_api import sync_playwright


def execute(urls: List[str]) -> List[Document]:
    print("Scraping anthropic docs..")
    main_urls = _get_all_urls(urls)
    main_urls = list(set(main_urls))
    docs: List[Document] = []
    for url in main_urls:
        soup = _scrape_html_with_playwright(url)
        tags = soup.find_all(class_="rm-Article")
        if len(tags):
            docs.append(Document(
                page_content=tags[0].text,
                metadata={"source": url}
            ))
        time.sleep(0.3)
    print("Scraped anthropic docs..")
    return docs


def _get_all_urls(urls: List[str]) -> List[str]:
    all_urls = []
    for url in urls:
        soup = _scrape_html_with_playwright(url)
        tags = soup.find_all("a", class_="rm-Sidebar-link")
        for tag in tags:
            if classes := tag.attrs.get("class"):
                if "side-nav-child" not in classes:
                    if href := tag.attrs.get("href"):
                        all_urls.append(urljoin(url, href))
    return all_urls


def _scrape_html_with_playwright(url) -> Optional[BeautifulSoup]:
    soup = None
    try:
        with sync_playwright() as p:
            browser = p.chromium.launch(headless=True)
            try:
                page = browser.new_page()
                page.goto(url)
                page_source = page.content()
                soup = BeautifulSoup(page_source, features="lxml")
            except:
                pass
            browser.close()
    except Exception as e:
        print(e)
    return soup



from fastapi import APIRouter
from fastapi import Body
from fastapi import Path

from opencopilot.logger import api_logger
from opencopilot.repository.conversation_history_repository import ConversationHistoryRepositoryLocal
from opencopilot.repository.conversation_logs_repository import ConversationLogsRepositoryLocal
from opencopilot.service.debug import message_debug_service
from opencopilot.service.debug.entities import EvaluationInput
from opencopilot.service.debug.entities import EvaluationResponse
from opencopilot.service.debug.entities import GetMessageDebugResponse
from opencopilot.service.evaluate import evaluation_service

TAG = "Debug"
router = APIRouter()
router.openapi_tags = [TAG]
router.title = "Debug router"

logger = api_logger.get()


@router.get(
    "/debug/{conversation_id}/{message_id}",
    tags=[TAG],
    summary="List custom copilots.",
    response_model=GetMessageDebugResponse
)
async def get_copilots(
        conversation_id: str = Path(
            ...,
            description="The ID of the conversation."),
        message_id: str = Path(
            ...,
            description="The ID of the response message."),
):
    history_repository = ConversationHistoryRepositoryLocal()
    logs_repository = ConversationLogsRepositoryLocal()

    return message_debug_service.execute(
        conversation_id,
        message_id,
        history_repository,
        logs_repository,
    )


@router.post(
    "/debug/evaluate",
    tags=[TAG],
    summary="Evaluate an LLM response.",
    response_model=EvaluationResponse,
)
async def evaluate(
        payload: EvaluationInput = Body(
            ...,
            description="Evaluated query.")
):
    return await evaluation_service.execute(payload)



import json
from datetime import datetime
from typing import Dict
from typing import List
from typing import Optional
from uuid import UUID

from langchain.schema import Document

from opencopilot.logger import api_logger
from opencopilot.domain.chat.entities import ChatContextInput

DEFAULT_CONTEXTS_DIR = "conversation_contexts"

logger = api_logger.get()


class ConversationUserContextRepositoryLocal:

    def __init__(self, contexts_dir: str = DEFAULT_CONTEXTS_DIR):
        self.contexts_dir = contexts_dir

    def get_context_documents(self, chat_id: UUID, count: Optional[int]) -> List[Document]:
        try:
            with open(self._get_file_path(chat_id), "r") as f:
                context = json.load(f)
            if not count or len(context) <= count:
                return self._to_documents(context)
            return self._to_documents(context[count * -1:])
        except:
            logger.error(f"Error loading conversation context, id: {str(chat_id)}")
        return []

    def get_context(self, chat_id: UUID) -> List[Dict]:
        context = []
        try:
            with open(self._get_file_path(chat_id), "r") as f:
                context = json.load(f)
        except:
            pass
        return context

    def _to_documents(self, context: List[Dict]) -> List[Document]:
        documents: List[Document] = []
        for i in context:
            documents.append(
                Document(
                    page_content=i.get("context", ""),
                    metadata={
                        "timestamp": i.get('timestamp', ''),
                        "source": "user_context"
                    }
                )
            )
        return documents

    def save_context(
            self,
            context_input: ChatContextInput
    ) -> None:
        # context = self.get_context(context_input.conversation_id)
        # context.append()
        self._write_file(
            chat_id=context_input.conversation_id,
            data=[
                {
                    "timestamp": datetime.now().isoformat(),
                    "context": context_input.context
                }
            ],
        )

    def _get_file_path(self, chat_id: UUID):
        return f"{self.contexts_dir}/{str(chat_id)}.json"

    def _write_file(self, chat_id: UUID, data):
        try:
            with open(self._get_file_path(chat_id), "w") as f:
                f.write(json.dumps(data, indent=4))
        except Exception as e:
            logger.warning(f"Failed to save context for chat {str(chat_id)}")



from typing import Optional, List

from pydantic import BaseModel
from pydantic import Field

from opencopilot.service.entities import ApiResponse


class ChatRequest(BaseModel):
    chat_id: str = Field(
        description="Chat id"
    )
    message: str
    response_message_id: Optional[str] = None
    email: Optional[str] = None


class ChatResponse(ApiResponse):
    chat_id: str = Field(
        description="Chat id"
    )

    message: str = Field(
        description="Chat output"
    )

    sources: List[str] = Field(
        default_factory=list,
        description="Sources"
    )

    class Config:
        schema_extra = {
            "example": {
                "response": "OK",
                "chat_id": "e91042aa-d53a-41eb-8884-67aa4947982d",
                "message": "I will use the 'search' command to find the weather in San Francisco."
            }
        }


class ChatFeedbackRequest(BaseModel):
    correctness: int = Field(
        description="How correct was the answer",
        ge=1,
        le=5,
        default=None
    )
    helpfulness: int = Field(
        description="How helpful was the answer",
        ge=1,
        le=5,
        default=None
    )
    easy_to_understand: int = Field(
        description="How easy to understand was the answer",
        ge=1,
        le=5,
        default=None
    )
    free_form_feedback: Optional[str] = Field(
        description="Free form user feedback for the answer"
    )


class CustomChatRequest(BaseModel):
    chat_id: str = Field(
        description="Chat id"
    )
    copilot_id: str = Field(
        description="Copilot id"
    )
    message: str
    response_message_id: Optional[str] = None

class ChatHistoryRequest(BaseModel):
    chat_id: str = Field(
        description="Chat id"
    )


class ChatHistoryItem(BaseModel):
    content: str
    timestamp: int


class ChatHistoryResponse(BaseModel):
    response: str
    chat_id: str = Field(
        description="Chat id"
    )
    messages: List[ChatHistoryItem] = Field(
        default_factory=list,
        description="Messages"
    )


class ChatContextRequest(BaseModel):
    context: str = Field(
        description="Additional context relevant to conversation"
    )
